@@ -1,22 +1,27 @@
+#![no_std]
 #![cfg_attr(test, deny(warnings))]
 #![deny(missing_docs)]
 
 //! Wrappers for total order on Floats.
 
 extern crate num_traits;
-extern crate unreachable;
+#[cfg(feature = "std")] extern crate std;
 
-use std::cmp::Ordering;
-use std::error::Error;
-use std::ops::{Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem,
+use core::cmp::Ordering;
+use core::ops::{Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem,
                RemAssign, Sub, SubAssign};
-use std::hash::{Hash, Hasher};
-use std::fmt;
-use std::io;
-use std::mem;
-use unreachable::unreachable;
-use num_traits::{Bounded, Float, FromPrimitive, Num, NumCast, One, Signed, ToPrimitive,
-                 Zero};
+use core::hash::{Hash, Hasher};
+use core::fmt;
+use core::mem;
+use core::hint::unreachable_unchecked;
+use core::iter::{Sum, Product};
+use core::str::FromStr;
+
+use num_traits::{Bounded, FromPrimitive, Num, NumCast, One, Signed, ToPrimitive, Zero};
+#[cfg(feature = "std")]
+use num_traits::Float;
+#[cfg(not(feature = "std"))]
+use num_traits::float::FloatCore as Float;
 
 /// A wrapper around Floats providing an implementation of Ord and Hash.
 ///
@@ -98,8 +103,6 @@ impl<T: Float> PartialEq for OrderedFloat<T> {
     fn eq(&self, other: &OrderedFloat<T>) -> bool {
         if self.as_ref().is_nan() {
             other.as_ref().is_nan()
-        } else if other.as_ref().is_nan() {
-            false
         } else {
             self.as_ref() == other.as_ref()
         }
@@ -157,6 +160,38 @@ impl<T: Float> DerefMut for OrderedFloat<T> {
 
 impl<T: Float> Eq for OrderedFloat<T> {}
 
+impl<T: Float> Add for OrderedFloat<T> {
+    type Output = Self;
+
+    fn add(self, other: Self) -> Self {
+        OrderedFloat(self.0 + other.0)
+    }
+}
+
+impl<T: Float> Sub for OrderedFloat<T> {
+    type Output = Self;
+
+    fn sub(self, other: Self) -> Self {
+        OrderedFloat(self.0 - other.0)
+    }
+}
+
+impl<T: Float> Mul for OrderedFloat<T> {
+    type Output = Self;
+
+    fn mul(self, other: Self) -> Self {
+        OrderedFloat(self.0 * other.0)
+    }
+}
+
+impl<T: Float> Div for OrderedFloat<T> {
+    type Output = Self;
+
+    fn div(self, other: Self) -> Self {
+        OrderedFloat(self.0 / other.0)
+    }
+}
+
 impl<T: Float> Bounded for OrderedFloat<T> {
     fn min_value() -> Self {
         OrderedFloat(T::min_value())
@@ -167,6 +202,37 @@ impl<T: Float> Bounded for OrderedFloat<T> {
     }
 }
 
+impl<T: Float + FromStr> FromStr for OrderedFloat<T> {
+    type Err = T::Err;
+
+    /// Convert a &str to `OrderedFloat`. Returns an error if the string fails to parse.
+    ///
+    /// ```
+    /// use ordered_float::OrderedFloat;
+    ///
+    /// assert!("-10".parse::<OrderedFloat<f32>>().is_ok());
+    /// assert!("abc".parse::<OrderedFloat<f32>>().is_err());
+    /// assert!("NaN".parse::<OrderedFloat<f32>>().is_ok());
+    /// ```
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        T::from_str(s).map(OrderedFloat)
+    }
+}
+
+impl<T: Float> Neg for OrderedFloat<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        OrderedFloat(-self.0)
+    }
+}
+
+impl<T: Float> Zero for OrderedFloat<T> {
+    fn zero() -> Self { OrderedFloat(T::zero()) }
+
+    fn is_zero(&self) -> bool { self.0.is_zero() }
+}
+
 /// A wrapper around Floats providing an implementation of Ord and Hash.
 ///
 /// A NaN value cannot be stored in this type.
@@ -195,15 +261,13 @@ impl<T: Float> NotNan<T> {
 
     /// Get the value out.
     pub fn into_inner(self) -> T {
-        let NotNan(val) = self;
-        val
+        self.0
     }
 }
 
 impl<T: Float> AsRef<T> for NotNan<T> {
     fn as_ref(&self) -> &T {
-        let NotNan(ref val) = *self;
-        val
+        &self.0
     }
 }
 
@@ -211,7 +275,7 @@ impl<T: Float> Ord for NotNan<T> {
     fn cmp(&self, other: &NotNan<T>) -> Ordering {
         match self.partial_cmp(&other) {
             Some(ord) => ord,
-            None => unsafe { unreachable() },
+            None => unsafe { unreachable_unchecked() },
         }
     }
 }
@@ -228,15 +292,15 @@ impl<T: Float + fmt::Display> fmt::Display for NotNan<T> {
     }
 }
 
-impl Into<f32> for NotNan<f32> {
-    fn into(self) -> f32 {
-        self.into_inner()
+impl From<NotNan<f32>> for f32 {
+    fn from(value: NotNan<f32>) -> Self {
+        value.into_inner()
     }
 }
 
-impl Into<f64> for NotNan<f64> {
-    fn into(self) -> f64 {
-        self.into_inner()
+impl From<NotNan<f64>> for f64 {
+    fn from(value: NotNan<f64>) -> Self {
+        value.into_inner()
     }
 }
 
@@ -245,8 +309,7 @@ impl Into<f64> for NotNan<f64> {
 /// Panics if the provided value is NaN or the computation results in NaN
 impl<T: Float> From<T> for NotNan<T> {
     fn from(v: T) -> Self {
-        assert!(!v.is_nan());
-        NotNan(v)
+        NotNan::new(v).expect("Tried to create a NotNan from a NaN")
     }
 }
 
@@ -267,7 +330,7 @@ impl<T: Float> Add for NotNan<T> {
     type Output = Self;
 
     fn add(self, other: Self) -> Self {
-        NotNan::new(self.0 + other.0).expect("Addition resulted in NaN")
+        self + other.0
     }
 }
 
@@ -278,44 +341,36 @@ impl<T: Float> Add<T> for NotNan<T> {
     type Output = Self;
 
     fn add(self, other: T) -> Self {
-        assert!(!other.is_nan());
         NotNan::new(self.0 + other).expect("Addition resulted in NaN")
     }
 }
 
-impl AddAssign for NotNan<f64> {
+impl<T: Float + AddAssign> AddAssign for NotNan<T> {
     fn add_assign(&mut self, other: Self) {
-        self.0 += other.0;
-        assert!(!self.0.is_nan(), "Addition resulted in NaN")
-    }
-}
-
-impl AddAssign for NotNan<f32> {
-    fn add_assign(&mut self, other: Self) {
-        self.0 += other.0;
-        assert!(!self.0.is_nan(), "Addition resulted in NaN")
-    }
-}
-
-/// Adds a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl AddAssign<f64> for NotNan<f64> {
-    fn add_assign(&mut self, other: f64) {
-        assert!(!other.is_nan());
-        self.0 += other;
-        assert!(!self.0.is_nan(), "Addition resulted in NaN")
+        *self += other.0;
     }
 }
 
 /// Adds a float directly.
 ///
 /// Panics if the provided value is NaN.
-impl AddAssign<f32> for NotNan<f32> {
-    fn add_assign(&mut self, other: f32) {
-        assert!(!other.is_nan());
+impl<T: Float + AddAssign> AddAssign<T> for NotNan<T> {
+    fn add_assign(&mut self, other: T) {
         self.0 += other;
-        assert!(!self.0.is_nan(), "Addition resulted in NaN")
+        assert!(!self.0.is_nan(), "Addition resulted in NaN");
+    }
+}
+
+
+impl<T: Float + Sum> Sum for NotNan<T> {
+    fn sum<I: Iterator<Item = NotNan<T>>>(iter: I) -> Self {
+        NotNan::new(iter.map(|v| v.0).sum()).expect("Sum resulted in NaN")
+    }
+}
+
+impl<'a, T: Float + Sum> Sum<&'a NotNan<T>> for NotNan<T> {
+    fn sum<I: Iterator<Item = &'a NotNan<T>>>(iter: I) -> Self {
+        iter.map(|v| *v).sum()
     }
 }
 
@@ -323,7 +378,7 @@ impl<T: Float> Sub for NotNan<T> {
     type Output = Self;
 
     fn sub(self, other: Self) -> Self {
-        NotNan::new(self.0 - other.0).expect("Subtraction resulted in NaN")
+        self - other.0
     }
 }
 
@@ -334,44 +389,23 @@ impl<T: Float> Sub<T> for NotNan<T> {
     type Output = Self;
 
     fn sub(self, other: T) -> Self {
-        assert!(!other.is_nan());
         NotNan::new(self.0 - other).expect("Subtraction resulted in NaN")
     }
 }
 
-impl SubAssign for NotNan<f64> {
+impl<T: Float + SubAssign> SubAssign for NotNan<T> {
     fn sub_assign(&mut self, other: Self) {
-        self.0 -= other.0;
-        assert!(!self.0.is_nan(), "Subtraction resulted in NaN")
-    }
-}
-
-impl SubAssign for NotNan<f32> {
-    fn sub_assign(&mut self, other: Self) {
-        self.0 -= other.0;
-        assert!(!self.0.is_nan(), "Subtraction resulted in NaN")
+        *self -= other.0
     }
 }
 
 /// Subtracts a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
-impl SubAssign<f64> for NotNan<f64> {
-    fn sub_assign(&mut self, other: f64) {
-        assert!(!other.is_nan());
+impl<T: Float + SubAssign> SubAssign<T> for NotNan<T> {
+    fn sub_assign(&mut self, other: T) {
         self.0 -= other;
-        assert!(!self.0.is_nan(), "Subtraction resulted in NaN")
-    }
-}
-
-/// Subtracts a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl SubAssign<f32> for NotNan<f32> {
-    fn sub_assign(&mut self, other: f32) {
-        assert!(!other.is_nan());
-        self.0 -= other;
-        assert!(!self.0.is_nan(), "Subtraction resulted in NaN")
+        assert!(!self.0.is_nan(), "Subtraction resulted in NaN");
     }
 }
 
@@ -379,7 +413,7 @@ impl<T: Float> Mul for NotNan<T> {
     type Output = Self;
 
     fn mul(self, other: Self) -> Self {
-        NotNan::new(self.0 * other.0).expect("Multiplication resulted in NaN")
+        self * other.0
     }
 }
 
@@ -390,43 +424,35 @@ impl<T: Float> Mul<T> for NotNan<T> {
     type Output = Self;
 
     fn mul(self, other: T) -> Self {
-        assert!(!other.is_nan());
         NotNan::new(self.0 * other).expect("Multiplication resulted in NaN")
     }
 }
 
-impl MulAssign for NotNan<f64> {
+impl<T: Float + MulAssign> MulAssign for NotNan<T> {
     fn mul_assign(&mut self, other: Self) {
-        self.0 *= other.0;
-        assert!(!self.0.is_nan(), "Multiplication resulted in NaN")
-    }
-}
-
-impl MulAssign for NotNan<f32> {
-    fn mul_assign(&mut self, other: Self) {
-        self.0 *= other.0;
-        assert!(!self.0.is_nan(), "Multiplication resulted in NaN")
+        *self *= other.0
     }
 }
 
 /// Multiplies a float directly.
 ///
 /// Panics if the provided value is NaN.
-impl MulAssign<f64> for NotNan<f64> {
-    fn mul_assign(&mut self, other: f64) {
-        assert!(!other.is_nan());
+impl<T: Float + MulAssign> MulAssign<T> for NotNan<T> {
+    fn mul_assign(&mut self, other: T) {
         self.0 *= other;
+        assert!(!self.0.is_nan(), "Multiplication resulted in NaN");
     }
 }
 
-/// Multiplies a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl MulAssign<f32> for NotNan<f32> {
-    fn mul_assign(&mut self, other: f32) {
-        assert!(!other.is_nan());
-        self.0 *= other;
-        assert!(!self.0.is_nan(), "Multiplication resulted in NaN")
+impl<T: Float + Product> Product for NotNan<T> {
+    fn product<I: Iterator<Item = NotNan<T>>>(iter: I) -> Self {
+        NotNan::new(iter.map(|v| v.0).product()).expect("Product resulted in NaN")
+    }
+}
+
+impl<'a, T: Float + Product> Product<&'a NotNan<T>> for NotNan<T> {
+    fn product<I: Iterator<Item = &'a NotNan<T>>>(iter: I) -> Self {
+        iter.map(|v| *v).product()
     }
 }
 
@@ -434,7 +460,7 @@ impl<T: Float> Div for NotNan<T> {
     type Output = Self;
 
     fn div(self, other: Self) -> Self {
-        NotNan::new(self.0 / other.0).expect("Division resulted in NaN")
+        self / other.0
     }
 }
 
@@ -445,44 +471,23 @@ impl<T: Float> Div<T> for NotNan<T> {
     type Output = Self;
 
     fn div(self, other: T) -> Self {
-        assert!(!other.is_nan());
         NotNan::new(self.0 / other).expect("Division resulted in NaN")
     }
 }
 
-impl DivAssign for NotNan<f64> {
+impl<T: Float + DivAssign> DivAssign for NotNan<T> {
     fn div_assign(&mut self, other: Self) {
-        self.0 /= other.0;
-        assert!(!self.0.is_nan(), "Division resulted in NaN")
-    }
-}
-
-impl DivAssign for NotNan<f32> {
-    fn div_assign(&mut self, other: Self) {
-        self.0 /= other.0;
-        assert!(!self.0.is_nan(), "Division resulted in NaN")
+        *self /= other.0;
     }
 }
 
 /// Divides a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
-impl DivAssign<f64> for NotNan<f64> {
-    fn div_assign(&mut self, other: f64) {
-        assert!(!other.is_nan());
+impl<T: Float + DivAssign> DivAssign<T> for NotNan<T> {
+    fn div_assign(&mut self, other: T) {
         self.0 /= other;
-        assert!(!self.0.is_nan(), "Division resulted in NaN")
-    }
-}
-
-/// Divides a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl DivAssign<f32> for NotNan<f32> {
-    fn div_assign(&mut self, other: f32) {
-        assert!(!other.is_nan());
-        self.0 /= other;
-        assert!(!self.0.is_nan(), "Division resulted in NaN")
+        assert!(!self.0.is_nan(), "Division resulted in NaN");
     }
 }
 
@@ -490,7 +495,7 @@ impl<T: Float> Rem for NotNan<T> {
     type Output = Self;
 
     fn rem(self, other: Self) -> Self {
-        NotNan::new(self.0 % other.0).expect("Rem resulted in NaN")
+        self % other.0
     }
 }
 
@@ -501,44 +506,23 @@ impl<T: Float> Rem<T> for NotNan<T> {
     type Output = Self;
 
     fn rem(self, other: T) -> Self {
-        assert!(!other.is_nan());
         NotNan::new(self.0 % other).expect("Rem resulted in NaN")
     }
 }
 
-impl RemAssign for NotNan<f64> {
+impl<T: Float + RemAssign> RemAssign for NotNan<T> {
     fn rem_assign(&mut self, other: Self) {
-        self.0 %= other.0;
-        assert!(!self.0.is_nan(), "Rem resulted in NaN")
-    }
-}
-
-impl RemAssign for NotNan<f32> {
-    fn rem_assign(&mut self, other: Self) {
-        self.0 %= other.0;
-        assert!(!self.0.is_nan(), "Rem resulted in NaN")
+        *self %= other.0
     }
 }
 
 /// Calculates `%=` with a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
-impl RemAssign<f64> for NotNan<f64> {
-    fn rem_assign(&mut self, other: f64) {
-        assert!(!other.is_nan());
+impl<T: Float + RemAssign> RemAssign<T> for NotNan<T> {
+    fn rem_assign(&mut self, other: T) {
         self.0 %= other;
-        assert!(!self.0.is_nan(), "Rem resulted in NaN")
-    }
-}
-
-/// Calculates `%=` with a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl RemAssign<f32> for NotNan<f32> {
-    fn rem_assign(&mut self, other: f32) {
-        assert!(!other.is_nan());
-        self.0 %= other;
-        assert!(!self.0.is_nan(), "Rem resulted in NaN")
+        assert!(!self.0.is_nan(), "Rem resulted in NaN");
     }
 }
 
@@ -546,7 +530,7 @@ impl<T: Float> Neg for NotNan<T> {
     type Output = Self;
 
     fn neg(self) -> Self {
-        NotNan::new(-self.0).expect("Negation resulted in NaN")
+        NotNan(-self.0)
     }
 }
 
@@ -554,7 +538,8 @@ impl<T: Float> Neg for NotNan<T> {
 #[derive(Copy, Clone, PartialEq, Eq, Debug)]
 pub struct FloatIsNan;
 
-impl Error for FloatIsNan {
+#[cfg(feature = "std")]
+impl std::error::Error for FloatIsNan {
     fn description(&self) -> &str {
         "NotNan constructed with NaN"
     }
@@ -566,9 +551,10 @@ impl fmt::Display for FloatIsNan {
     }
 }
 
-impl Into<io::Error> for FloatIsNan {
-    fn into(self) -> io::Error {
-        io::Error::new(io::ErrorKind::InvalidInput, self)
+#[cfg(feature = "std")]
+impl Into<std::io::Error> for FloatIsNan {
+    fn into(self) -> std::io::Error {
+        std::io::Error::new(std::io::ErrorKind::InvalidInput, self)
     }
 }
 
@@ -613,6 +599,26 @@ impl<T: Float> Bounded for NotNan<T> {
     }
 }
 
+impl<T: Float + FromStr> FromStr for NotNan<T> {
+    type Err = ParseNotNanError<T::Err>;
+
+    /// Convert a &str to `NotNan`. Returns an error if the string fails to parse,
+    /// or if the resulting value is NaN
+    ///
+    /// ```
+    /// use ordered_float::NotNan;
+    ///
+    /// assert!("-10".parse::<NotNan<f32>>().is_ok());
+    /// assert!("abc".parse::<NotNan<f32>>().is_err());
+    /// assert!("NaN".parse::<NotNan<f32>>().is_err());
+    /// ```
+    fn from_str(src: &str) -> Result<Self, Self::Err> {
+        src.parse()
+            .map_err(ParseNotNanError::ParseFloatError)
+            .and_then(|f| NotNan::new(f).map_err(|_| ParseNotNanError::IsNaN))
+    }
+}
+
 impl<T: Float + FromPrimitive> FromPrimitive for NotNan<T> {
     fn from_i64(n: i64) -> Option<Self> { T::from_i64(n).and_then(|n| NotNan::new(n).ok()) }
     fn from_u64(n: u64) -> Option<Self> { T::from_u64(n).and_then(|n| NotNan::new(n).ok()) }
@@ -654,14 +660,18 @@ pub enum ParseNotNanError<E> {
     IsNaN,
 }
 
-impl<E: fmt::Debug> Error for ParseNotNanError<E> {
+#[cfg(feature = "std")]
+impl<E: fmt::Debug> std::error::Error for ParseNotNanError<E> {
     fn description(&self) -> &str {
         return "Error parsing a not-NaN floating point value";
     }
+
+    // TODO: add an implementation of cause(). This will be breaking because it requires E: Error.
 }
 
 impl<E: fmt::Debug> fmt::Display for ParseNotNanError<E> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        // TODO: replace this with a human readable fmt. Will require E: Display.
         <Self as fmt::Debug>::fmt(self, f)
     }
 }
@@ -680,7 +690,7 @@ impl<T: Float + Signed> Signed for NotNan<T> {
     fn abs(&self) -> Self { NotNan(self.0.abs()) }
 
     fn abs_sub(&self, other: &Self) -> Self {
-        NotNan::new(self.0.abs_sub(other.0)).expect("Subtraction resulted in NaN")
+        NotNan::new(Signed::abs_sub(&self.0, &other.0)).expect("Subtraction resulted in NaN")
     }
 
     fn signum(&self) -> Self { NotNan(self.0.signum()) }
@@ -700,8 +710,11 @@ mod impl_serde {
     use self::serde::{Serialize, Serializer, Deserialize, Deserializer};
     use self::serde::de::{Error, Unexpected};
     use super::{OrderedFloat, NotNan};
+    #[cfg(feature = "std")]
     use num_traits::Float;
-    use std::f64;
+    #[cfg(not(feature = "std"))]
+    use num_traits::float::FloatCore as Float;
+    use core::f64;
 
     #[cfg(test)]
     extern crate serde_test;
